from spryx_core import NOT_GIVEN, is_given
from spryx_http import SpryxAsyncClient


class Files:
    def __init__(self, client: SpryxAsyncClient):
        self._client = client

    async def presign_upload(
        self,
        file_name: str,
        content_type: str,
    ) -> dict:
        """Get a pre-signed URL for file upload."""
        payload = {
            "file_name": file_name,
            "content_type": content_type,
        }

        return await self._client.post(
            "/files/presign/upload",
            json=payload,
        )

    async def add_file(
        self,
        object_key: str,
    ) -> dict:
        """Add a file using the object key from a successful upload."""
        payload = {
            "object_key": object_key,
        }

        return await self._client.post("/files", json=payload)

    async def get_file(
        self,
        file_id: str,
    ) -> dict:
        """Retrieve a file by ID."""
        return await self._client.get(f"/files/{file_id}") 


from spryx_http import SpryxAsyncClient

from spryx_agents.resources.agents import Agents
from spryx_agents.resources.chats import Chats
from spryx_agents.resources.credential_types import CredentialTypes
from spryx_agents.resources.credentials import Credentials
from spryx_agents.resources.embedding_usages import EmbeddingUsages
from spryx_agents.resources.files import Files
from spryx_agents.resources.integrations import Integrations
from spryx_agents.resources.vector_stores import VectorStores


class SpryxAgents(SpryxAsyncClient):
    def __init__(
        self,
        application_id: str,
        application_secret: str,
        base_url: str = "https://dev-agents.spryx.ai",
        iam_base_url: str = "https://dev-iam.spryx.ai",
    ):
        super().__init__(
            base_url=base_url,
            iam_base_url=iam_base_url,
            application_id=application_id,
            application_secret=application_secret,
        )

        self.agents = Agents(self)
        self.chats = Chats(self)
        self.credential_types = CredentialTypes(self)
        self.credentials = Credentials(self)
        self.files = Files(self)
        self.integrations = Integrations(self)
        self.vector_stores = VectorStores(self)
        self.embedding_usages = EmbeddingUsages(self)



from spryx_core import NOT_GIVEN, is_given
from spryx_http import SpryxAsyncClient


class Chats:
    def __init__(self, client: SpryxAsyncClient):
        self._client = client

    async def list_chats(
        self,
        page: int = 1,
        limit: int = 10,
        order: str = "asc",
    ) -> dict:
        """List all chats."""
        return await self._client.get(
            "/chats/",
            params={"page": page, "limit": limit, "order": order},
        )

    async def get_chat(
        self,
        chat_id: str,
    ) -> dict:
        """Retrieve a chat by ID."""
        return await self._client.get(f"/chats/{chat_id}")

    async def update_chat(
        self,
        chat_id: str,
        name: str,
        description: str,
        private: bool,
    ) -> dict:
        """Update a chat."""
        payload = {
            "name": name,
            "description": description,
            "private": private,
        }

        return await self._client.put(
            f"/chats/{chat_id}",
            json=payload,
        )

    async def list_messages(
        self,
        chat_id: str,
        page: int = 1,
        limit: int = 10,
        order: str = "asc",
    ) -> dict:
        """List all messages in a chat."""
        return await self._client.get(
            f"/chats/{chat_id}/messages",
            params={"page": page, "limit": limit, "order": order},
        ) 


from spryx_core import NOT_GIVEN, is_given
from spryx_http import SpryxAsyncClient


class CredentialTypes:
    def __init__(self, client: SpryxAsyncClient):
        self._client = client

    async def list_credential_types(
        self,
        type_name: str = NOT_GIVEN,
    ) -> dict:
        """List all credential types."""
        params = {}
        
        if is_given(type_name):
            params["type_name"] = type_name
            
        return await self._client.get(
            "/credential-types/",
            params=params,
        ) 


from spryx_core import NOT_GIVEN, is_given
from spryx_http import SpryxAsyncClient

from spryx_agents.types.agent import LlmModel


class Agents:
    def __init__(self, client: SpryxAsyncClient):
        self._client = client
        self._base_url = client.base_url

    async def list_agents(
        self,
        page: int = 1,
        limit: int = 10,
        order: str = "asc",
        organization_id: str = NOT_GIVEN,
    ) -> dict:
        """List all agents."""
        return await self._client.get(
            f"{self._base_url}agents",
            params={"page": page, "limit": limit, "order": order},
            headers={"x-organization-id": organization_id},
        )

    async def create_agent(
        self,
        name: str,
        instructions: str,
        temperature: float = 1.0,
        top_p: float = 1.0,
        vector_store_id: str = NOT_GIVEN,
        input_guardrail_instructions: str = NOT_GIVEN,
        output_guardrail_instructions: str = NOT_GIVEN,
        llm_model: LlmModel = LlmModel.GPT_4O_MINI,
        description: str = NOT_GIVEN,
        icon: str = NOT_GIVEN,
        color: str = NOT_GIVEN,
        icon_type: str = NOT_GIVEN,
        is_active: bool = True,
        metadata: dict = None,
        resources: list = None,
        handoff_rules: list = None,
        organization_id: str = NOT_GIVEN,
    ) -> dict:
        """Create a new agent."""
        payload = {
            "name": name,
            "instructions": instructions,
            "temperature": temperature,
            "top_p": top_p,
            "llm_model": llm_model.value,
            "is_active": is_active,
        }

        if is_given(vector_store_id):
            payload["vector_store_id"] = vector_store_id
        
        if is_given(input_guardrail_instructions):
            payload["input_guardrail_instructions"] = input_guardrail_instructions
        
        if is_given(output_guardrail_instructions):
            payload["output_guardrail_instructions"] = output_guardrail_instructions
        
        if is_given(description):
            payload["description"] = description
        
        if is_given(icon):
            payload["icon"] = icon
        
        if is_given(color):
            payload["color"] = color
        
        if is_given(icon_type):
            payload["icon_type"] = icon_type

        if resources:
            payload["resources"] = resources
        
        if handoff_rules:
            payload["handoff_rules"] = handoff_rules
            
        if metadata:
            payload["metadata"] = metadata

        return await self._client.post(f"{self._base_url}agents", json=payload, headers={"x-organization-id": organization_id})

    async def get_agent(
        self,
        agent_id: str,
        organization_id: str = NOT_GIVEN,
    ) -> dict:
        """Retrieve a specific agent by ID."""
        return await self._client.get(f"{self._base_url}agents/{agent_id}", headers={"x-organization-id": organization_id})

    async def update_agent(
        self,
        agent_id: str,
        name: str,
        instructions: str,
        temperature: float = 1.0,
        top_p: float = 1.0,
        vector_store_id: str = NOT_GIVEN,
        input_guardrail_instructions: str = NOT_GIVEN,
        output_guardrail_instructions: str = NOT_GIVEN,
        llm_model: LlmModel = LlmModel.GPT_4O_MINI,
        description: str = NOT_GIVEN,
        icon: str = NOT_GIVEN,
        color: str = NOT_GIVEN,
        icon_type: str = NOT_GIVEN,
        is_active: bool = True,
        metadata: dict = None,
        resources: list = None,
        handoff_rules: list = None,
        organization_id: str = NOT_GIVEN,
    ) -> dict:
        """Update an existing agent."""
        payload = {
            "name": name,
            "instructions": instructions,
            "temperature": temperature,
            "top_p": top_p,
            "llm_model": llm_model.value,
            "is_active": is_active,
        }

        if is_given(vector_store_id):
            payload["vector_store_id"] = vector_store_id
        
        if is_given(input_guardrail_instructions):
            payload["input_guardrail_instructions"] = input_guardrail_instructions
        
        if is_given(output_guardrail_instructions):
            payload["output_guardrail_instructions"] = output_guardrail_instructions
        
        if is_given(description):
            payload["description"] = description
        
        if is_given(icon):
            payload["icon"] = icon
        
        if is_given(color):
            payload["color"] = color
        
        if is_given(icon_type):
            payload["icon_type"] = icon_type

        if resources:
            payload["resources"] = resources
        
        if handoff_rules:
            payload["handoff_rules"] = handoff_rules
            
        if metadata:
            payload["metadata"] = metadata

        return await self._client.put(f"{self._base_url}agents/{agent_id}", json=payload, headers={"x-organization-id": organization_id})

    async def invoke_agent(
        self,
        agent_id: str,
        user_prompt: list,
        chat_id: str = NOT_GIVEN,
        organization_id: str = NOT_GIVEN,
    ) -> dict:
        """Invoke an agent with a user prompt."""
        payload = {
            "user_prompt": user_prompt,
        }

        if is_given(chat_id):
            payload["chat_id"] = chat_id

        return await self._client.post(f"{self._base_url}agents/{agent_id}/invoke", json=payload, headers={"x-organization-id": organization_id})
